from ffxivcalc.Jobs.Base_Spell import DOTSpell, ManaRequirement, empty
from ffxivcalc.Jobs.Healer.Healer_Spell import SageSpell
from ffxivcalc.Jobs.Healer.Sage.Sage_Spell import SageSpell
import copy

Lock = 0.75


#Requirement

def PhysisRequirement(Player, Spell):
    return Player.PhysisCD <= 0, Player.PhysisCD

def EukrasianDosisRequirement(Player, Spell):
    return Player.Eukrasia, -1

def PneumaRequirement(Player, Spell):
    return Player.PneumaCD <= 0, Player.PneumaCD

def PhlegmaRequirement(Player, Spell):
    return Player.PhlegmaStack > 0, Player.PhlegmaCD

def AddersgallRequirement(Player, Enemy):
    return Player.AddersgallStack > 0, -1

def KrasisRequirement(Player, Spell):
    return Player.KrasisCD <= 0, Player.KrasisCD

def PanhaimaRequirement(Player, Spell):
    return Player.PanhaimaCD <= 0, Player.PanhaimaCD

def HolosRequirement(Player, Spell):
    return Player.HolosCD <= 0, Player.HolosCD

def RhizomataRequirement(Player, Spell):
    return Player.RhizomataCD <= 0, Player.RhizomataCD

def HaimaRequirement(Player, Spell):
    return Player.HaimaCD <= 0, Player.HaimaCD

def TaurocholeRequirement(Player, Spell):
    return Player.TaurocholeCD <= 0, Player.TaurocholeCD

def PepsiRequirement(Player, Spell):
    return Player.PepsiCD <= 0, Player.PepsiCD

def ZoeRequirement(Player, Spell):
    return Player.ZoeCD <= 0, Player.ZoeCD

def IxocholeRequirement(Player, Spell):
    return Player.IxocholeCD <= 0, Player.IxocholeCD

def KeracholeRequirement(Player, Spell):
    return Player.KeracholeCD <= 0, Player.KeracholeCD

def IcarusRequirement(Player, Spell):
    return Player.IcarusCD <= 0, Player.IcarusCD

def SoteriaRequirement(Player, Spell):
    return Player.SoteriaCD <= 0, Player.SoteriaCD

def ToxikonRequirement(Player, Spell):
    return Player.AdderstingStack > 0, -1

#Apply

def ApplyPhysis(Player, Enemy):
    Player.PhysisCD = 60

def ApplyToxikon(Player, Enemy):
    Player.AdderstingStack -= 1

def ApplyEukrasianDiagnosis(Player, Enemy):
    Player.AdderstingStack = min(3, Player.AdderstingStack + 1)
    #We assume we can cast a Toxikon after casting this shield by assuming it breaks

def ApplyDiagnosis(Player, Enemy):
    Player.AdderstingStack = min(3, Player.AdderstingStack + 1) #Will be assumed each time we do it gives a stack

def ApplyKrasis(Player, Enemy):
    Player.KrasisCD = 60

def ApplyPanhaima(Player, Enemy):
    Player.AdderstingStack = min(3, Player.AdderstingStack + 1)
    Player.PanhaimaCD = 120

def ApplyHolos(Player, Enemy):
    Player.HolosCD = 120

def ApplyRhizomata(Player, Enemy):
    Player.RhizomataCD = 90
    Player.AddersgallStack = min(3, Player.AddersgallStack + 1)

def Apply(Player, Enemy):
    Player.CD = 0

def ApplyHaima(Player, Enemy):
    Player.AdderstingStack = min(3, Player.AdderstingStack + 1)
    Player.HaimaCD = 120

def ApplyTaurochole(Player, Enemy):
    Player.TaurocholeCD = 45
    Player.AddersgallStack -= 1

def ApplyPepsi(Player, Enemy):
    Player.PepsiCD = 30

def ApplyZoe(Player, Enemy):
    Player.ZoeCD = 90

def ApplyKerachole(Player, Enemy):
    Player.KeracholeCD = 30
    Player.AddersgallStack -= 1

def ApplyIxochole(Player, Enemy):
    Player.IxocholeCD = 30
    Player.AddersgallStack -= 1

def ApplyIcarus(Player, Enemy):
    Player.IcarusCD = 45

def ApplySoteria(Player, Enemy):
    Player.SoteriaCD = 90

def ApplyDruochole(Player, Enemy):
    Player.AddersgallStack -= 1

def ApplyEukrasia(Player, Enemy):
    Player.Eukrasia = True

def ApplyPneuma(Player, Enemy):
    Player.PneumaCD = 120

def ApplyPhlegma(Player, Enemy):
    if Player.PhlegmaStack == 2:
        Player.EffectCDList.append(PhlegmaStackCheck)
        Player.FlegmaCD = 40
    Player.PhlegmaStack -= 1

def ApplyEukrasian(Player, Enemy):
    Player.Eukrasia = False
    if Player.Eukrasian == None:
        Player.Eukrasian = copy.deepcopy(EukrasianDOT)
        Player.EffectCDList.append(EukrasianDOTCheck)
        Player.DOTList.append(Player.Eukrasian)
    else : Player.Eukrasian.resetBuffSnapshot() # If already applied reset snapshot
    Player.EukrasianTimer = 30



#Check

def PhlegmaStackCheck(Player, Enemy):
    if Player.PhlegmaTimer <= 0:
        if Player.PhlegmaStack == 1:
            Player.EffectToRemove.append(PhlegmaStackCheck)
        else:
            Player.PhlegmaTimer = 45
        Player.PhlegmaStack +=1



def EukrasianDOTCheck(Player, Enemy):
    if Player.EukrasianTimer <= 0:
        Player.EffectToRemove.append(EukrasianDOTCheck)
        Player.DOTList.remove(Player.Eukrasian)
        Player.Eukrasian = None


#Damage GCD
Dosis = SageSpell(24312, True, 1.5, 2.5, 330, 400, empty, [ManaRequirement], type = 1)
EukrasianDosis = SageSpell(24314, True, 0, 1.5, 0, 400, ApplyEukrasian, [ManaRequirement, EukrasianDosisRequirement], type = 1)
EukrasianDOT = DOTSpell(-12, 75, False)
Phlegma = SageSpell(24313, True, Lock, 2.5, 490, 400, ApplyPhlegma, [PhlegmaRequirement], type = 1)
Pneuma = SageSpell(24318, True, 1.5, 2.5, 330, 700, ApplyPneuma, [ManaRequirement, PneumaRequirement], type = 1)
Toxikon = SageSpell(24316, True, 0, 2.5, 330, 0, ApplyToxikon, [ToxikonRequirement], type = 1)
Dyskrasia = SageSpell(24315, True, 0, 2.5, 170, 400, empty, [ManaRequirement], type = 1)
#Healing GCD
Egeiro = SageSpell(24287, True, 8, 2.5, 0, 2400, empty, [ManaRequirement], type = 1)
Prognosis = SageSpell(24286, True, 2, 2.5, 0, 800, empty, [ManaRequirement], type = 1, AOEHeal=True)
EukrasianPrognosis = SageSpell(24292, True, 0, 1.5, 0, 900, empty, [ManaRequirement], type = 1, AOEHeal=True)
Diagnosis = SageSpell(24284, True, 1.5, 2.5, 0, 400, ApplyDiagnosis, [ManaRequirement], type = 1)
EukrasianDiagnosis = SageSpell(24291, True, 1.5, 1.5, 0, 900, ApplyEukrasianDiagnosis, [ManaRequirement], type = 1, TargetHeal=True)

Eukrasia = SageSpell(24290, True, 1, 1, 0, 0, ApplyEukrasia, [], type = 1) #Since only 1 sec CD, no real need to put a requirement
#Healing oGCD
Krasis = SageSpell(24317, False, 0, 0, 0, 0, ApplyKrasis, [KrasisRequirement])
Panhaima = SageSpell(24311, False, 0, 0, 0, 0, ApplyPanhaima, [PanhaimaRequirement], AOEHeal=True)
Holos = SageSpell(24310, False, 0, 0, 0, 0, ApplyHolos, [HolosRequirement], AOEHeal=True)
Rhizomata = SageSpell(24309, False, 0, 0, 0, 0, ApplyRhizomata, [RhizomataRequirement])
Haima = SageSpell(24305, False, 0, 0, 0, 0, ApplyHaima, [HaimaRequirement])
Taurochole = SageSpell(24303, False, 0, 0, 0, 0, ApplyTaurochole, [TaurocholeRequirement,AddersgallRequirement], TargetHeal=True)
Pepsi = SageSpell(24301, False, 0, 0, 0, 0, ApplyPepsi, [PepsiRequirement])
Zoe = SageSpell(24300, False, 0, 0, 0, 0, ApplyZoe, [ZoeRequirement]) #https://www.google.com/url?sa=i&url=https%3A%2F%2Fwww.leagueoflegends.com%2Fen-pl%2Fchampions%2Fzoe%2F&psig=AOvVaw2X9QcnXQ_CGp3xMY8MLua9&ust=1654295724885000&source=images&cd=vfe&ved=0CAwQjRxqFwoTCJjl4oPqj_gCFQAAAAAdAAAAABAD
Ixochole = SageSpell(24299, False, 0, 0, 0, 0, ApplyIxochole, [IxocholeRequirement,AddersgallRequirement], AOEHeal=True)
Kerachole = SageSpell(24298, False, 0, 0, 0, 0, ApplyKerachole, [KeracholeRequirement,AddersgallRequirement])
Icarus = SageSpell(24295, False, 0, 0, 0, 0, ApplyIcarus, [IcarusRequirement])
Soteria = SageSpell(24294, False, 0, 0, 0, 0, ApplySoteria, [SoteriaRequirement])
Druochole = SageSpell(24296, False, 0, 0, 0, 0, ApplyDruochole, [AddersgallRequirement], TargetHeal=True)
Kardia = SageSpell(24285, False, 0, 0, 0, 0, empty, [])
Physis = SageSpell(24302, False, 0, 0, 0, 0, ApplyPhysis, [PhysisRequirement])

SageAbility = {
24287 : Egeiro,
24312 : Dosis,
24314 : EukrasianDosis,
24316 : Toxikon,
24315 : Dyskrasia,
24313 : Phlegma,
24285 : Kardia,
24290 : Eukrasia,
24284 : Diagnosis,
24291 : EukrasianDiagnosis,
24286 : Prognosis,
24292 : EukrasianPrognosis,
24302 : Physis,
24294 : Soteria,
24295 : Icarus,
24296 : Druochole,
24298 : Kerachole,
24299 : Ixochole,
24300 : Zoe,
24301 : Pepsi,
24303 : Taurochole,
24305 : Haima,
24309 : Rhizomata,
24310 : Holos,
24311 : Panhaima,
24317 : Krasis,
24318 : Pneuma

}


#########################################
########## MACHINIST SPELL  #############
#########################################


import copy
from ffxivcalc.Jobs.Base_Spell import DOTSpell, Queen_AA, empty, WaitAbility
from ffxivcalc.Jobs.Ranged.Ranged_Spell import MachinistSpell
from ffxivcalc.Jobs.Player import Pet

Lock = 0

#Special

def AddGauge(Player, Battery, Heat):
    Player.BatteryGauge = min(100, Player.BatteryGauge + Battery)
    Player.HeatGauge = min(100, Player.HeatGauge + Heat)

def RemoveGauge(Player, Battery, Heat):
    Player.BatteryGauge = max(0, Player.BatteryGauge - Battery)
    Player.HeatGauge = max(0, Player.HeatGauge - Heat)

#Requirement

def DetonatorRequirement(Player, Spell):
    return Player.WildFireTimer > 0, -1

def DismantleRequirement(Player, Spell):
    return Player.DismantleCD <= 0, Player.DismantleCD

def OverheatedRequirement(Player, Spell):
    return Player.HyperchargeStack > 0, -1 #True if Overheated

def WildFireRequirement(Player, Spell):
    return Player.WildFireCD <= 0, Player.WildFireCD

def AirAnchorRequirement(Player, Spell):
    return Player.AirAnchorCD <= 0, Player.AirAnchorCD

def BarrelStabilizerRequirement(Player, Spell):
    return Player.BarrelStabilizerCD <= 0, Player.BarrelStabilizerCD

def HyperchargeRequirement(Player, Spell):
    return Player.HyperchargeCD <= 0 and Player.HeatGauge >= 50, -1

def ReassembleRequirement(Player, Spell):
    return Player.ReassembleStack > 0, -1

def GaussRoundRequirement(Player, Spell):
    return Player.GaussRoundStack > 0, Player.GaussRoundCD

def RicochetRequirement(Player, Spell):
    return Player.RicochetStack > 0, Player.RicochetCD

def DrillRequirement(Player, Spell):
    return Player.DrillCD <= 0, Player.DrillCD

def OverdriveRequirement(Player, Spell):
    return Player.Overdrive, -1

def ChainSawRequirement(Player, Spell):
    return Player.ChainSawCD <= 0, Player.ChainSawCD

def AutomatonRequirement(Player, Spell):
    return (not Player.Overdrive) and Player.BatteryGauge >= 50, -1

def TacticianRequirement(Player, Spell):
    return Player.TacticianCD <= 0, Player.TacticianCD

#Apply

def ApplyDetonator(Player, Spell):
    Player.WildFireTimer = 0

def ApplyDismantle(Player, Enemy):
    Player.DismantleCD = 120

def ApplyTactician(Player, Enemy):
    Player.TacticianCD = 90

def ApplyScattergun(Player, Enemy):
    AddGauge(Player, 0, 10)

def ApplyBioblaster(Player, Enemy):
    ApplyDrill(Player, Enemy)
    if Player.BioblasterDOT == None:
        Player.BioblasterDOT = copy.deepcopy(BioblasterDOT)
        Player.DOTList.append(Player.BioblasterDOT)
        Player.BioblasterDOTTimer = 15
        Player.EffectCDList.append(BioblasterDOTCheck)
    Player.BioblasterDOTTimer = 15

def ApplyWildFire(Player, Enemy):
    Player.WildFireCD = 120
    Player.WildFireTimer = 10
    Player.EffectList.append(WildFireEffect)
    Player.EffectCDList.append(WildFireCheck)

def ApplyAirAnchor(Player, Enemy):
    AddGauge(Player, 20, 0)
    Player.AirAnchorCD = 40 * Player.WeaponskillReduction

def ApplyBarrelStabilizer(Player, Enemy):
    AddGauge(Player, 0, 50)
    Player.BarrelStabilizerCD = 120

def ApplyHeatBlast(Player, Enemy):
    if Player.GaussRoundStack < 3 : Player.GaussRoundCD -= 15
    if Player.RicochetStack < 3 : Player.RicochetCD -= 15

def ApplyHypercharge(Player, Enemy):
    Player.HyperchargeStack = 5
    Player.HyperchargeCD = 10
    RemoveGauge(Player, 0, 50)#cost
    Player.EffectList.append(HyperchargeEffect)
    Player.EffectCDList.append(HyperchargeCheck)

def ApplyReassemble(Player, Enemy):
    if Player.ReassembleStack == 2:
        Player.EffectCDList.append(ReassembleStackCheck)
        Player.ReassembleCD = 55
    Player.ReassembleStack -= 1
    Player.Reassemble = True
    
def ApplyGaussRound(Player, Enemy):
    if Player.GaussRoundStack == 3:
        Player.EffectCDList.append(GaussRoundStackCheck)
        Player.GaussRoundCD = 30
    Player.GaussRoundStack -= 1

def ApplyRicochet(Player, Enemy):
    if Player.RicochetStack == 3:
        Player.EffectCDList.append(RicochetStackCheck)
        Player.RicochetCD = 30
    Player.RicochetStack -= 1

def ApplyDrill(Player, Enemy):
    Player.DrillCD = 20 * Player.WeaponskillReduction

def ApplyOverdrive(Player, Enemy):
    Player.Overdrive = False
    Player.QueenTimer = 0

def ApplyChainSaw(Player, Enemy):
    AddGauge(Player, 20, 0)
    Player.ChainSawCD = 60 * Player.WeaponskillReduction

def ApplyAutomaton(Player, Enemy):
    Player.QueenStartUpTimer = 5
    Player.EffectCDList.append(QueenStartUpCheck)

def SummonQueen(Player, Enemy):
    #input("SummoningQueen at : " + str(Player.CurrentFight.TimeStamp))
    Player.AutomatonQueenCD = 6

    QueenTimer = (Player.BatteryGauge - 50)/5 + 10 #Queen timer by linearly extrapolating 10 sec base + extra battery Gauge
    #Queen Timer = (ExtraBatteryGauge) / BatteryPerSec - StartUpTimer + BaseTimer = Battery/5 - 5 + 10
    Player.BatteryGauge = 0
    if Player.Queen == None : Player.Queen = Pet(Player)#Creating new queen
    else: Player.Queen.ResetStat() # Reseting stat on the summon
    Player.QueenTimer = QueenTimer - 3 # Setting Queen Timer, gives last 3 seconds to perform finisher moves
    #Will have to depend on battery Gauge
    #Timer is set at 10 so we can have 2 GCD to do finisher move if reaches before
    Player.Queen.QueenOnField = True
    Player.EffectCDList.append(QueenCheck)
    Player.Queen.ActionSet.append(Queen_AA)
    Player.Queen.ActionSet.append(WaitAbility(QueenTimer - 3)) #Gives 3 last sec to do finishing move
    Player.Queen.TrueLock = False #Delocking the queen if she was in a locked state, would happen is resummoned

def ApplyCollider(Queen, Enemy):#Called on queen
    Queen.Master.QueenOnField = False

#Combo Actions

def ApplySplitShot(Player, Enemy):
    AddGauge(Player, 0, 5)
    if not (SplitShotEffect in Player.EffectList) : Player.EffectList.append(SplitShotEffect)

def ApplySlugShot(Player, Enemy):
    pass

def ApplyCleanShot(Player, Enemy):
    pass

#Effect

def WildFireEffect(Player, Spell):
    if isinstance(Spell, MachinistSpell) and Spell.Weaponskill : Player.WildFireStack = min(6, Player.WildFireStack + 1) # Max of 6

def HyperchargeEffect(Player, Spell):
    if isinstance(Spell, MachinistSpell) and Spell.Weaponskill : 
        Spell.Potency += 20
        Player.RemoveHyperchargeStack = True

#Combo Actions effect

def SplitShotEffect(Player, Spell):
    if Spell.id == SlugShot.id:

        Spell.Potency += 180
        Player.EffectToRemove.append(SplitShotEffect)
        if not (SlugShotEffect in Player.EffectList) : Player.EffectToAdd.append(SlugShotEffect)
        AddGauge(Player, 0, 5)
    elif Spell.id == SplitShot.id or Spell.id == CleanShot.id:
        Player.EffectToRemove.append(SplitShotEffect)

def SlugShotEffect(Player, Spell):
    if Spell.id == CleanShot.id:
        Spell.Potency += 260
        Player.EffectToRemove.append(SlugShotEffect)
        AddGauge(Player, 10, 5)
    elif Spell.id == SplitShot.id or Spell.id == SlugShot.id:
        Player.EffectToRemove.append(SlugShotEffect)


#Check

def QueenStartUpCheck(Player, Enemy):
    if Player.QueenStartUpTimer <= 0:
        SummonQueen(Player, Enemy) #Waits for 5 sec then summons the queen
        Player.EffectToRemove.append(QueenStartUpCheck)

def FlamethrowerDOTCheck(Player, Enemy):
    if Player.FlamethrowerDOTTimer <= 0:
        Player.DOTList.remove(Player.FlamethrowerDOT)
        Player.FlamethrowerDOT = None
        Player.EffectToRemove.append(FlamethrowerDOTCheck)

def BioblasterDOTCheck(Player, Enemy):
    if Player.BioblasterDOTTimer <= 0:
        Player.DOTList.remove(Player.BioblasterDOT)
        Player.BioblasterDOT = None
        Player.EffectToRemove.append(BioblasterDOTCheck)

def WildFireCheck(Player, Enemy):
    if Player.WildFireTimer <= 0:
        WildFireOff = MachinistSpell(-2878, False, 0, 0, 240 * Player.WildFireStack, 0, empty, [], False)
        # Gives same ID as wildfire action so it shows in the log file
        #Temporary Spell that will be put in front of the Queue
        Player.ActionSet.insert(Player.NextSpell+1, WildFireOff) #Insert in queue, will be instantly executed
        Player.EffectList.remove(WildFireEffect)
        Player.EffectToRemove.append(WildFireCheck)
        Player.WildFireStack = 0

def HyperchargeCheck(Player, Enemy):

    if Player.RemoveHyperchargeStack:
        Player.RemoveHyperchargeStack = False
        Player.HyperchargeStack -= 1

    if Player.HyperchargeStack == 0:
        Player.EffectList.remove(HyperchargeEffect)
        Player.EffectToRemove.append(HyperchargeCheck)

def ReassembleStackCheck(Player, Enemy):
    if Player.ReassembleCD <= 0:
        if Player.ReassembleStack == 1:
            Player.EffectToRemove.append(ReassembleStackCheck)
        else:
            Player.ReassembleCD = 55
        Player.ReassembleStack +=1

def GaussRoundStackCheck(Player, Enemy):
    if Player.GaussRoundCD <= 0:
        if Player.GaussRoundStack == 2:
            Player.EffectToRemove.append(GaussRoundStackCheck)
            Player.GaussRoundCD = 0
        else:
                             # Since Heatblast removes 15 seconds from the CD, it must be able to carry
                             # over to the next stack.
                             # I had a little wiggle game to account for rounding/small errors
                             # and otherwise I simply add 30 seconds to the reduced CD.
                             # The reason it can be negative is because heatblast removes a flat
                             # 15 seconds to the timer.
            if Player.GaussRoundCD < -0.05 :
                Player.GaussRoundCD += 30
            else : 
                Player.GaussRoundCD = 30
        Player.GaussRoundStack +=1

def RicochetStackCheck(Player, Enemy):
    if Player.RicochetCD <= 0:
        if Player.RicochetStack == 2:
            Player.EffectToRemove.append(RicochetStackCheck)
            Player.RicochetCD = 0
        else:
                             # Since Heatblast removes 15 seconds from the CD, it must be able to carry
                             # over to the next stack.
                             # I had a little wiggle game to account for rounding/small errors
                             # and otherwise I simply add 30 seconds to the reduced CD.
                             # The reason it can be negative is because heatblast removes a flat
                             # 15 seconds to the timer.
            if Player.RicochetCD < -0.05 :
                Player.RicochetCD += 30
            else : 
                Player.RicochetCD = 30
        Player.RicochetStack +=1

def QueenCheck(Player, Enemy): # This will turn the queen off and make her perform the finisher moves
    if Player.QueenTimer <= 0: 
        Player.Overdrive = False
        Player.QueenOnField = False
        Player.QueenTimer = 0
        Player.Queen.TrueLock = False #Delocking the Queen so she can perform these two abilities
        Player.Queen.ActionSet.insert(Player.NextSpell+1,Bunker)
        Player.Queen.ActionSet.insert(Player.NextSpell+2,Collider)
        Player.EffectToRemove.append(QueenCheck)
        Player.Queen.EffectCDList.append(QueenAACheck)


def QueenAACheck(Player, Enemy):
    if Player.TrueLock:#This function will be called on the Queen once it has finished Collider, it will get rid of AA's
        #It checks for when the Queen is done
        Player.DOTList = [] #Reset DOTList
        Player.EffectToRemove.append(QueenAACheck)




Dismantle = MachinistSpell(2887, False, 0, 0, 0, 0, ApplyDismantle, [DismantleRequirement], False)
Wildfire = MachinistSpell(2878, False, 0, Lock, 0, 0, ApplyWildFire, [WildFireRequirement], False)
Detonator = MachinistSpell(1111111, False, 0, 0, 0, 0, ApplyDetonator, [DetonatorRequirement], False)
AirAnchor = MachinistSpell(16500, True, 0, 2.5, 600, 0, ApplyAirAnchor, [AirAnchorRequirement], True, type = 2)
BarrelStabilizer = MachinistSpell(7414, False, 0, Lock, 0, 0, ApplyBarrelStabilizer, [BarrelStabilizerRequirement], False)
HeatBlast = MachinistSpell(7410, True, Lock, 1.5, 200, 0, ApplyHeatBlast, [OverheatedRequirement], True, type = 2)
Hypercharge = MachinistSpell(17209, False, 0, Lock, 0, 0, ApplyHypercharge, [HyperchargeRequirement], False)
Reassemble = MachinistSpell(2876, False, 0, Lock, 0, 0, ApplyReassemble, [ReassembleRequirement], False)
GaussRound = MachinistSpell(2874, False, 0, Lock, 130, 0, ApplyGaussRound, [GaussRoundRequirement], False)
Ricochet = MachinistSpell(2890, False, 0, Lock, 130, 0, ApplyRicochet, [RicochetRequirement], False)
Drill = MachinistSpell(16498, True, 0, 2.5, 600, 0, ApplyDrill, [DrillRequirement], True, type = 2)
ChainSaw = MachinistSpell(25788, True, 0, 2.5, 600, 0, ApplyChainSaw, [ChainSawRequirement], True, type = 2)
Tactician = MachinistSpell(16889, False, 0, 0, 0, 0, ApplyTactician, [TacticianRequirement], False)
#Combo Action

SplitShot = MachinistSpell(7411, True, Lock, 2.5, 200, 0, ApplySplitShot, [], True, type = 2)
SlugShot = MachinistSpell(7412, True, Lock, 2.5, 120, 0, ApplySlugShot, [], True , type = 2)
CleanShot = MachinistSpell(7413, True, Lock, 2.5, 120, 0, ApplyCleanShot, [], True, type = 2)


#AOE GCD
AutoCrossbow = MachinistSpell(16497, True, 0, 1.5, 140, 0, empty, [OverheatedRequirement], True, type = 2)
Scattergun = MachinistSpell(25768, True, 0, 2.5, 150, 0, ApplyScattergun, [], True, type = 2)
Bioblaster = MachinistSpell(16499, True, 0, 0, 50, 0, ApplyBioblaster, [DrillRequirement], True, type = 2) #Shares CD with Drill
BioblasterDOT = DOTSpell(-2, 50, True)
FlamethrowerDOT = DOTSpell(-3, 80, True)
def Flamethrower(time):
    #This function will apply a dot for the specified duration, and lock the player for this duration



    def FlamethrowerRequirement(Player, Spell):
        return time > 10 and Player.FlamethrowerCD <= 0, Player.FlamethrowerCD

    def ApplyFlamethrower(Player, Enemy):
        Player.FlamethrowerCD = 60
        Player.FlamethrowerDOTTimer = time
        Player.FlamethrowerDOT = copy.deepcopy(FlamethrowerDOT)
        Player.DOTList.append(Player.FlamethrowerDOT)
        Player.EffectCDList.append(FlamethrowerDOTCheck)

    return MachinistSpell(7418, True, time, time, 0, 0, ApplyFlamethrower, [FlamethrowerRequirement], False)

#Queen's Ability

#These abilities will write into the Queen's ability list.
#If they are not done the queen will do them automatically
Automaton = MachinistSpell(16501, False, 0, Lock, 0, 0, ApplyAutomaton, [], False)
Overdrive = MachinistSpell(16502, False, 0, Lock, 0, 0, ApplyOverdrive, [], False)
#These will be casted by the machinist, so they have no damage. Their only effect is to add into Queen's Queue
Bunker = MachinistSpell(15, True, 0, 2.5, 680, 0, empty, [], False)   #Triggered by Overdrive
Collider = MachinistSpell(16, True, 0 , 2.5, 780, 0, ApplyCollider, [], False)  #Spell Queen will cast

MachinistAbility = {
7411 : SplitShot, 
7412 : SlugShot, 
25788 : ChainSaw,
25768 : Scattergun, 
17209 : Hypercharge, 
16889 : Tactician, 
16502 : Overdrive, 
16501 : Automaton,
16500 : AirAnchor, 
16499 : Bioblaster, 
16498 : Drill,
16497 : AutoCrossbow, 
7418 : Flamethrower(2.5), 
7414 : BarrelStabilizer, 
7413 : CleanShot, 
7410 : HeatBlast, 
2878 : Wildfire,
1111111 : Detonator,
2874 : GaussRound, 
2890 : Ricochet,
2876 : Reassemble,
2887 : Dismantle
}




"""
This code was taken from a comment in : https://github.com/IAmPythagoras/FFXIV-Combat-Simulator/issues/17
and was written by https://github.com/Alex-Ueki @apollo#3810
and modified to fit the needs I have.
"""
# Installed versions I use
# coreapi==2.3.3
# coreapi-cli==1.0.9
import coreapi
import logging

main_logging = logging.getLogger("ffxivcalc")
etro_logging = main_logging.getChild("etroAPI")

def get_gearset_data(set_id: str) -> dict:
    """
    Gets the gearset data (stats) via an id. Allows url too (handles it for free)
    set_id : str -> URL of the set from etro
    """
    # Handles urls by checking for webpage name and splitting
    logging.debug("Requesting gearset info from etro.gg, set_id : " + str(set_id))
    try:
        cleaned_set_id = set_id.split("/")[-1] if "etro.gg/gearset" in set_id else set_id # Cleans the URL
        client = coreapi.Client()
        data = client .action(
            client.get("https://etro.gg/api/docs/"),
            ["gearsets", "read"],
            params={
                "id": cleaned_set_id,
            },
        )


        # Since the first 8 data points are the stats. That's all we are interested in.
        # We will not use all of them since we will filter through. But this contains all we need

        stats = {
            data["totalParams"][i]["name"] : data["totalParams"][i]["value"] 
            for i in range(len(data["totalParams"]))
        }

        # Will now change the keys' name

        return {
            "MainStat" : data["totalParams"][0]["value"] , # Always first value
            "WD" : stats["Weapon Damage"] if "Weapon Damage" in stats.keys() else 0,
            "Det" : stats["DET"],
            "Ten" : stats["TEN"] if "TEN" in stats.keys() else 400,
            "SS" : stats["SPS"] if "SPS" in stats.keys() else 400,
            "SkS" : stats["SkS"] if "SkS" in stats.keys() else 400,
            "Crit" : stats["CRT"],
            "DH" : stats["DH"],
            "Piety" : stats["Piety"] if "Piety" in stats.keys() else 390
        }
    except:
                             # If error during import (wrong URL, server error, etc.)
                             # it returns None
        return None


#########################################
########## DARK KNIGHT SKILLS ###########
#########################################
from ffxivcalc.Jobs.Base_Spell import DOTSpell, Potion, buff, empty
import copy
from ffxivcalc.Jobs.Player import Pet, Shield, MitBuff
from ffxivcalc.Jobs.Tank.Tank_Spell import BigMitRequirement, ApplyBigMit, DRKSkill
Lock = 0

def BloodRequirement(Player, Spell):
    if Player.DeliriumStacks > 0 and (Spell.id == Bloodspiller.id or Spell.id == Quietus.id):
        Spell.BloodCost = 0
        Player.DeliriumStacks -= 1
        return True, -1
                             # Applying cost and checking if works or not.
    Player.Blood -= 50
    if Player.Blood >= 0:
        return True, -1
    return False, -1

def EdgeShadowRequirement(Player, Spell):
    if Player.EdgeShadowCD <= 0 :
        if Player.DarkArts:
            Spell.ManaCost = 0
            Player.DarkArts = False
            return True, -1
        elif Player.Mana >= Spell.ManaCost:
            Player.Mana -= Spell.ManaCost
            return True, -1
        return True, -1
    return False, -1

def BloodWeaponRequirement(Player, Spell):
    return Player.BloodWeaponCD <= 0, Player.BloodWeaponCD

def DeliriumRequirement(Player, Spell):
    return Player.DeliriumCD <= 0, Player.DeliriumCD

def CarveSpitRequirement(Player, Spell):
    return Player.CarveSpitCD <= 0, Player.CarveSpitCD

def AbyssalDrainRequirement(Player, Spell):
    return Player.AbyssalDrainCD <= 0, Player.AbyssalDrainCD

def SaltedEarthRequirement(Player, Spell):
    return Player.SaltedEarthCD <= 0, Player.SaltedEarthCD

def SaltDarknessRequirement(Player, Spell):
    return Player.SaltedEarthTimer > 0, -1

def ShadowbringerRequirement(Player, Spell):
    return Player.DarksideTimer > 0 and Player.ShadowbringerCharges > 0, -1


def PlungeRequirement(Player, Spell):
    return Player.PlungeCharges > 0, Player.PlungeCD

def TBNRequirement(Player, Spell):
    if Player.Mana >= Spell.ManaCost:
        Player.Mana -= Spell.ManaCost
        return True, -1
    return True, -1

def LivingDeadRequirement(Player, Spell):
    return Player.LivingDeadCD <= 0, Player.LivingDeadCD

def DarkMindRequirement(Player, Spell):
    return Player.DarkMindCD <= 0, Player.DarkMindCD

def DarkMissionaryRequirement(Player, Spell):
    return Player.DarkMissionaryCD <= 0, Player.DarkMissionaryCD

def OblationRequirement(Player, Spell):
    return Player.OblationStack > 0, Player.OblationCD

#Effect functions that persist after action use

def BloodWeaponEffect(Player, Spell):
    if Spell.GCD:
        Player.Mana = min(Player.Mana + 600, 10000)
        Player.Blood = min(100, Player.Blood + 10)
        Player.BloodWeaponStacks -= 1

def DeliriumEffect(Player, Spell):
    if Spell.id == Bloodspiller.id:
        Player.Mana = min(Player.Mana + 200, 10000)
    elif Spell.id == Quietus.id:
        Player.Mana = min(Player.Mana + 500, 10000)

def HardSlashEffect(Player, Spell):
    if Spell.id == SyphonStrike.id:
        Spell.Potency += 140
        Player.Mana = min(Player.Mana + 600, 10000)
        
        Player.EffectToRemove.append(HardSlashEffect)
        if not SyphonStrikeEffect in Player.EffectList : Player.EffectList.append(SyphonStrikeEffect)

def SyphonStrikeEffect(Player, Spell):
    if Spell.id == Souleater.id:
        Spell.Potency += 220
        Player.Blood = min(100, Player.Blood + 20)
        Player.EffectToRemove.append(SyphonStrikeEffect)

#Cooldown checks to remove effect and restore charges


def OblationStackCheck(Player, Spell):
    if Player.OblationCD <= 0:
        if Player.OblationStack == 1:
            Player.EffectToRemove.append(OblationStackCheck)
        else:
            Player.OblationCD = 60
        Player.OblationStack += 1

def BloodWeaponCheck(Player, Spell):
    if Player.BloodWeaponTimer <= 0 or Player.BloodWeaponStacks == 0:
        Player.EffectList.remove(BloodWeaponEffect)
        Player.EffectToRemove.append(BloodWeaponCheck)

def DeliriumCheck(Player, Spell):
    if Player.DeliriumTimer <= 0 or Player.DeliriumStacks <= 0:
        Player.EffectList.remove(DeliriumEffect)
        Player.EffectToRemove.append(DeliriumCheck)

def SaltedEarthCheck(Player, Spell):
    if Player.SaltedEarthTimer <= 0:
        Player.DOTList.remove(Player.SaltedEarthDOT)
        Player.SaltedEarthDOT = None
        Player.SaltedEarthTimer = 0
        Player.EffectToRemove.append(SaltedEarthCheck)

def CheckShadowbringerCharge(Player, Enemy):
    if Player.ShadowbringerCD <= 0:
        if Player.ShadowbringerCharges == 0:
            Player.ShadowbringerCD = 30
        if Player.ShadowbringerCharges == 1:
            Player.EffectToRemove.append(CheckShadowbringerCharge)
        Player.ShadowbringerCharges +=1

def CheckPlungeCharge(Player, Enemy):
    if Player.PlungeCD <= 0:
        if Player.PlungeCharges == 0:
            Player.PlungeCD = 30
        if Player.PlungeCharges == 1:
            Player.EffectToRemove.append(CheckPlungeCharge)
        Player.PlungeCharges +=1

def UnleashCombo(Player, Spell):
    if Spell.id == StalwartSoul.id:
        Spell.Potency += 40
        Player.Blood = min(100, Player.Blood + 20)


#Apply effects that happen upon action use

def ApplyLivingDead(Player, Enemy):
    Player.LivingDeadCD = 300

    Player.InvulnTimer = 10

def ApplyUnleash(Player, Spell):
    if not (UnleashCombo in Player.EffectList) : Player.EffectList.append(UnleashCombo)

def ApplyHardSlashEffect(Player, Spell):
    if not (HardSlashEffect in Player.EffectList) : Player.EffectList.append(HardSlashEffect)

def ApplySyphonEffect(Player, Spell):
    pass

def ApplyBloodWeaponEffect(Player, Spell):
    Player.BloodWeaponCD = 60                     
    Player.EffectList.append(BloodWeaponEffect)
    Player.BloodWeaponStacks = 5
    Player.BloodWeaponTimer = 15
    Player.EffectCDList.append(BloodWeaponCheck)

def ApplyDeliriumEffect(Player, Spell):
    Player.DeliriumCD = 60
    Player.EffectList.append(DeliriumEffect)
    Player.DeliriumStacks = 3
    Player.DeliriumTimer = 30 
    Player.EffectCDList.append(DeliriumCheck)

def ApplyEdgeShadowEffect(Player, Spell):
    Player.DarksideTimer = min(60, Player.DarksideTimer + 30)
    if not (CheckEdgeShadow in Player.EffectCDList):
        Player.buffList.append(EdgeShadowBuff)
        Player.EffectCDList.append(CheckEdgeShadow)

def CheckEdgeShadow(Player, Enemy):
    if Player.DarksideTimer <= 0:
        Player.buffList.remove(EdgeShadowBuff)
        Player.EffectCDList.remove(CheckEdgeShadow)

def ApplyCarveSpitEffect(Player, Spell):
    Player.CarveSpitCD = 60
    Player.AbyssalDrainCD = 60
    Player.Mana = min(Player.Mana + 600, 10000)

def ApplyAbyssalDrainEffect(Player, Spell):
    Player.AbyssalDrainCD = 60
    Player.CarveSpitCD = 60

def ApplySaltedEarth(Player, Spell):
    Player.SaltedEarthCD = 90
    Player.SaltedEarthTimer = 15
    if Player.SaltedEarthDOT == None:
        Player.SaltedEarthDOT = copy.deepcopy(SaltedEarthDOT)
        Player.DOTList.append(Player.SaltedEarthDOT)
        Player.EffectCDList.append(SaltedEarthCheck)
    else : Player.SaltedEarthDOT.resetBuffSnapshot()

def ApplySaltDarknessEffect(Player, Spell):
    Player.SaltDarknessCD = 15

def SpendShadowbringer(Player, Spell):
    if Player.ShadowbringerCharges == 2 :
        Player.ShadowbringerCD = 60
    Player.ShadowbringerCharges -= 1
    Player.EffectCDList.append(CheckShadowbringerCharge)

def SummonLivingShadow(Player, Spell):

    if Player.Pet == None : 
        Pet(Player) # Creating a pet
    else:
        Player.Pet.ResetStat()

    Actions = [PDelay, PAbyssalDrain, PPlunge, PShadowbringer, PEdgeShadow, PBloodspiller, PCarveSpit] #This should take 20 seconds
    Player.Pet.ActionSet = Actions
    Player.Pet.TrueLock = False #Delocking
    Player.Pet.NextSpell = 0 #Reseting

def SpendPlunge(Player,Spell):
    if Player.PlungeCharges == 2 :
        Player.PlungeCD = 30
    Player.PlungeCharges -= 1
    Player.EffectCDList.append(CheckPlungeCharge)

def ApplyDarkArts(Player, Spell):
    Player.DarkArts = True


def ApplyDarkMind(Player, Spell):
    Player.DarkMindCD = 60

    DarkMindMit = MitBuff(0.8, 10, Player, MagicMit=True)
    Player.MitBuffList.append(DarkMindMit)

def ApplyDarkMissionary(Player, Enemy):
    Player.DarkMissionaryCD = 90

    # Will give all players in the fight 10% magic mit

    for player in Player.CurrentFight.PlayerList:
        player.MitBuffList.append(MitBuff(0.9, 15, player, MagicMit=True))


def ApplyOblation(Player, Enemy):
    if Player.OblationStack == 2:
        Player.EffectCDList.append(OblationStackCheck)
        Player.OblationCD = 60
    Player.OblationStack -= 1

#List of Weaponskills and Spells used by a Dark Knight Player.
DRKGCD = 2.5         #GCD speed
Lock = 0            #Fixed value for animation lock.

HardSlash = DRKSkill(3617, True, Lock, DRKGCD, 170, 0, 0, ApplyHardSlashEffect, [], type = 2)
SyphonStrike = DRKSkill(3623, True, Lock, DRKGCD, 120, 0, 0, empty, [], type = 2)
Souleater = DRKSkill(3632, True, Lock, DRKGCD, 120, 0, 0, empty, [], type = 2)
Bloodspiller = DRKSkill(7392, True, Lock, DRKGCD, 500, 0, 50, empty, [BloodRequirement], type = 2)
Quietus = DRKSkill(5, True, Lock, DRKGCD, 200, 0, 50, empty, [BloodRequirement], type = 2)
Unmend = DRKSkill(3624, True, Lock, 2.50, 150, 0, 0, empty, [], type = 1)

#List of Buffs used by a Dark Knight Player.

BloodWeapon = DRKSkill(3625, False, Lock, 0, 0, 0, 0, ApplyBloodWeaponEffect, [BloodWeaponRequirement])
Delirium = DRKSkill(7390, False, Lock, 0, 0, 0, 0, ApplyDeliriumEffect, [DeliriumRequirement])

#List of Abilities used by a Dark Knight Player.

EdgeShadow = DRKSkill(16470, False, Lock, 0, 460, 3000, 0, ApplyEdgeShadowEffect, [EdgeShadowRequirement])
FloodShadow = DRKSkill(10, False, Lock, 0, 160, 3000, 0, ApplyEdgeShadowEffect, [EdgeShadowRequirement])
CarveSpit = DRKSkill(3643, False, Lock, 0, 510, 0, 0, ApplyCarveSpitEffect, [CarveSpitRequirement])
AbyssalDrain = DRKSkill(12, False, Lock, 0, 150, 0, 0, ApplyAbyssalDrainEffect, [AbyssalDrainRequirement])
SaltedEarth = DRKSkill(3639, False, Lock, 0, 0, 0, 0, ApplySaltedEarth, [SaltedEarthRequirement]) #Ground target DOT, ticks once upon placement.
SaltedEarthDOT = DOTSpell(-14, 50, True, isGround=True)
SaltDarkness = DRKSkill(25755, False, Lock, 0, 500, 0, 0, empty, [SaltDarknessRequirement])
Shadowbringer = DRKSkill(25757, False, Lock, 0, 600, 0, 0, SpendShadowbringer, [ShadowbringerRequirement])
LivingShadow = DRKSkill(16472, False, Lock, 0, 0, 0, 50, SummonLivingShadow, [BloodRequirement])
Plunge = DRKSkill(3640, False, Lock, 0, 150, 0, 0, SpendPlunge, [PlungeRequirement])

#AOE GCD
Unleash = DRKSkill(3621, True, 0, 2.5, 120, 0, 0, ApplyUnleash, [], type = 1)
StalwartSoul = DRKSkill(16468, True, 0, 2.5, 100, 0, 0, empty, [], type = 1)
#List of Abilities performed by Living Shadow.

PAbyssalDrain = DRKSkill(22, True, 0.5, 2.33, 350, 0, 0, empty, [])
PPlunge = DRKSkill(23, True, 0.5, 2.33, 350, 0, 0, empty, [])
PShadowbringer = DRKSkill(25, True, 0.5, 2.33, 500, 0, 0, empty, [])
PEdgeShadow = DRKSkill(26, True, 0.5, 2.33, 350, 0, 0, empty, [])
PBloodspiller = DRKSkill(27, True, 0.5, 2.33, 350, 0, 0, empty, [])
PCarveSpit = DRKSkill(28, True, 0.5, 2.33, 350, 0, 0, empty, [])
PDelay = DRKSkill(29, True, 0, 6, 0, 0, 0, empty, [])    #6s animation before it starts attacking.
#Living Shadow 

#Mit
LivingDead = DRKSkill(3638, False, 0, 0, 0, 0, 0, ApplyLivingDead, [LivingDeadRequirement])
DarkMind = DRKSkill(3634, False, 0, 0, 0, 0, 0, ApplyDarkMind, [DarkMindRequirement])
DarkMissionary = DRKSkill(16471, False, 0, 0, 0, 0, 0, ApplyDarkMissionary, [DarkMissionaryRequirement])
ShadowWall = DRKSkill(3636, False, 0, 0, 0, 0, 0, ApplyBigMit, [BigMitRequirement])

def TBN(Target):
    """This function returns a DRKSpell object corresponding to TBN
    that targets the given Target

    Args:
        Target (Player): Player targetted by TBN. Can be the DRK itself
    """

    def ApplyTBN(Player, Enemy):
        ApplyDarkArts(Player, Enemy)
                             # Refreshing targetID
        TBNSpell.TargetID = Target.playerID

        TBNShield = Shield(int(Target.MaxHP * 0.25), 7, Target)
        Target.ShieldList.insert(0, TBNShield) # Inserting shield at first since TBN has prio

    TBNSpell = DRKSkill(7393, False, Lock, 0, 0, 3000, 0, ApplyTBN, [TBNRequirement])     #Simply makes the next EdgeShadow free for now.
    TBNSpell.TargetID = Target.playerID
    TBNSpell.TargetPlayerObject = Target
    return TBNSpell


def Oblation(Target):
    """This function returns a DRK spell object corresponding to oblation that targets a player


    Args:
        Target (Player): Player targetted by Oblation. Can be the DRK itself
    """

    def OblationCheck(Player, Enemy):
        if Player.OblationTimer <= 0:
            Target.MagicMitigation /= 0.9
            Target.PhysicalMitigation /= 0.9
            Player.EffectToRemove.append(OblationCheck)            

    def ApplyTargetOblation(Player, Enemy):
        ApplyOblation(Player, Enemy) # Applies any cooldown and stuff to the player

        Target.OblationTimer = 10
        if not (OblationCheck in Target.EffectCDList):
            Target.MagicMitigation *= 0.9
            Target.PhysicalMitigation *= 0.9
            Target.EffectCDList.append(OblationCheck)
    
    OblationSpell = DRKSkill(25754, False, 0, 0, 0, 0, 0, ApplyTargetOblation, [OblationRequirement])
    OblationSpell.TargetID = Target.playerID
    return OblationSpell

        


#buff
EdgeShadowBuff = buff(1.1,name="Darkside")

# Maps Ability IDs from FFlogs to Skill objects
# Not exhaustive: If an ability is missing, check log for id and add it into the mapping.
DarkKnightAbility = {
    3617: HardSlash,
    3623: SyphonStrike,
    3624: Unmend,
    3625: BloodWeapon,
    3632: Souleater,
    3634: DarkMind,
    3636: ShadowWall,
    3638: LivingDead,
    3639: SaltedEarth,
    3640: Plunge,
    3643: CarveSpit,
    7390: Delirium,
    7392: Bloodspiller,
    7393: TBN,
    16470: EdgeShadow,
    16471: DarkMissionary,
    16472: LivingShadow,
    25754: Oblation,
    25755: SaltDarkness,
    25757: Shadowbringer,
    10 : FloodShadow,
    3621 : Unleash,
    16468 : StalwartSoul,
    5 : Quietus,
    12 : AbyssalDrain,
    34590541: Potion  # This is assumed to be strength pot grade 6
}


from time import time
from ffxivcalc.helperCode.helper_math import roundDown

class ProgressBar:
    """
    This class can be used to represent a progress bar that the user will see.
    You cannot use the __init__ method of the class, you must use the class method init()
    in order to get an iter(ProgressBar). To increment it simply use next(ProgressBar) and it will
    display itself.
    total : int -> total number of iterations
    name : str -> Name of the progress bar
    currentState : str -> Last generated frame of the progress bar
    extraBuffer : -> place to write the PB too
    """
    def __init__(self):
        self.total = 0
        self.currentProgress = 0
        self.name = ""
        self.currentState = ""
        self.extraBuffer = None
        self.showBar = True
        
                             # Timing related fields
        self.iterationAverage = 0 
        self.sumIterations = 0
        self.lastIterTime = 0
        self.lenLoadBar = 0

    def getCurrentState(self):
        return self.currentState
    
    def setShowBar(self, val):
        self.showBar = val

    def setExtraBuffer(self, buff):
        self.extraBuffer = buff

    def __iter__(self):
        self.currentProgress = -1
        self.name = self.name
        return self

    def __next__(self):
        """
        This function updates the progress of the bar and updates the display of the bar.
        """
        self.currentProgress += 1
        percent = int((self.currentProgress/self.total)*1000)/10


        curTimeIteration = time() - self.lastIterTime
        self.lastIterTime = time()
        self.sumIterations += curTimeIteration
        self.iterationAverage = round(self.sumIterations/self.currentProgress,1) if self.currentProgress > 0 else self.sumIterations

        predictedTime = round(self.iterationAverage * (self.total-self.currentProgress),1)

        bar = "❚" * int(percent) + "-" * (100-int(percent))

        loadBar = "\r"+ self.name +" |"+bar+"| " + ((str(percent) + " %") if self.currentProgress > 0 else "") + " ETA : " + str(predictedTime) + "s"
        self.currentState = loadBar
        if self.extraBuffer != None: self.extraBuffer['pb'] = loadBar
        

                             # This will remove characters that are not supposed to be there anymore
        if len(loadBar) > self.lenLoadBar: self.lenLoadBar = len(loadBar)
        for i in range(len(loadBar), self.lenLoadBar): loadBar += " "
                             # only print on screen if showBar is true
        if self.showBar: print(loadBar, end="\r")
        if (self.total - self.currentProgress == 0) and self.showBar:print()
        return self
    
    def complete(self):
        """
        This function will terminate the progress bar
        """
        self.currentProgress = self.total-1
        next(self)

    def setName(self, newName : str):
        """Sets the name of the progress bar to the newly given name
        """
        self.name = newName
    
    @classmethod
    def init(self, total : int, name : str, showBar : bool = True, extraBuffer = None):
        """
        This class method returns an iterator of the progress bar.

        total : int -> Total amount of iterations
        name : string -> name of loading bar (that is displayed)
        showBar : bool -> If true displays the progress bar using print()
        extraBuffer -> Dictionnary in which the pb view will be written at the key 'pb'
        """
        newProgressBar = ProgressBar()
        newProgressBar.total = total
        newProgressBar.name = name
        newProgressBar.lastIterTime = time()
        iterator = iter(newProgressBar)
        iterator.setExtraBuffer(extraBuffer)
        iterator.setShowBar(showBar)
        next(iterator)
        return iterator
    

if __name__ == "__main__":
    pass